package solutions.bitbadger.documents.scala

import org.scalatest.funspec.AnyFunSpec
import solutions.bitbadger.documents.scala.support.{ByteIdClass, IntIdClass, LongIdClass, ShortIdClass, StringIdClass}
import solutions.bitbadger.documents.{AutoId, DocumentException}

class AutoIdSpec extends AnyFunSpec {

  describe("generateUUID") {
    it("generates a UUID string") {
      assert(32 == AutoId.generateUUID().length)
    }
  }

  describe("generateRandomString") {
    it("generates a random hex character string of an even length") {
      assert(8 == AutoId.generateRandomString(8).length)
    }
    it("generates a random hex character string of an odd length") {
      assert(11 == AutoId.generateRandomString(11).length)
    }
    it("generates different random hex character strings") {
      val result1 = AutoId.generateRandomString(16)
      val result2 = AutoId.generateRandomString(16)
      assert(result1 != result2)
    }
  }

  describe("needsAutoId") {
    it("fails for null document") {
      assertThrows[DocumentException] { AutoId.needsAutoId(AutoId.DISABLED, null, "id") }
    }
    it("fails for missing ID property") {
      assertThrows[DocumentException] { AutoId.needsAutoId(AutoId.UUID, IntIdClass(0), "Id") }
    }
    it("returns false if disabled") {
      assert(!AutoId.needsAutoId(AutoId.DISABLED, "", ""))
    }
    it("returns true for Number strategy and byte ID of 0") {
      assert(AutoId.needsAutoId(AutoId.NUMBER, ByteIdClass(0), "id"))
    }
    it("returns false for Number strategy and byte ID of non-0") {
      assert(!AutoId.needsAutoId(AutoId.NUMBER, ByteIdClass(77), "id"))
    }
    it("returns true for Number strategy and short ID of 0") {
      assert(AutoId.needsAutoId(AutoId.NUMBER, ShortIdClass(0), "id"))
    }
    it("returns false for Number strategy and short ID of non-0") {
      assert(!AutoId.needsAutoId(AutoId.NUMBER, ShortIdClass(31), "id"))
    }
    it("returns true for Number strategy and int ID of 0") {
      assert(AutoId.needsAutoId(AutoId.NUMBER, IntIdClass(0), "id"))
    }
    it("returns false for Number strategy and int ID of non-0") {
      assert(!AutoId.needsAutoId(AutoId.NUMBER, IntIdClass(6), "id"))
    }
    it("returns true for Number strategy and long ID of 0") {
      assert(AutoId.needsAutoId(AutoId.NUMBER, LongIdClass(0), "id"))
    }
    it("returns false for Number strategy and long ID of non-0") {
      assert(!AutoId.needsAutoId(AutoId.NUMBER, LongIdClass(2), "id"))
    }
    it("fails for Number strategy and non-number ID") {
      assertThrows[DocumentException] { AutoId.needsAutoId(AutoId.NUMBER, StringIdClass(""), "id") }
    }
    it("returns true for UUID strategy and blank ID") {
      assert(AutoId.needsAutoId(AutoId.UUID, StringIdClass(""), "id"))
    }
    it("returns false for UUID strategy and non-blank ID") {
      assert(!AutoId.needsAutoId(AutoId.UUID, StringIdClass("howdy"), "id"))
    }
    it("fails for UUID strategy and non-string ID") {
      assertThrows[DocumentException] { AutoId.needsAutoId(AutoId.UUID, IntIdClass(5), "id") }
    }
    it("returns true for Random String strategy and blank ID") {
      assert(AutoId.needsAutoId(AutoId.RANDOM_STRING, StringIdClass(""), "id"))
    }
    it("returns false for Random String strategy and non-blank ID") {
      assert(!AutoId.needsAutoId(AutoId.RANDOM_STRING, StringIdClass("full"), "id"))
    }
    it("fails for Random String strategy and non-string ID") {
      assertThrows[DocumentException] { AutoId.needsAutoId(AutoId.RANDOM_STRING, ShortIdClass(55), "id") }
    }
  }
}
